% Example MATLAB code to calculate river catchment properties using TopoToolbox 

% National-scale geodatabase of catchment characteristics in the Philippines for river management applications
% February 2023 - richard.boothroyd@glasgow.ac.uk
% Manuscript DOI: 10.1371/journal.pone.0281933 
% Data DOI: http://dx.doi.org/10.5525/gla.researchdata.1396   

% Example uses the Tigum catchment, the Philippines.
% Requires TopoToolbox - https://github.com/wschwanghart/topotoolbox 
% Requires Hypsometric Integral - https://uk.mathworks.com/matlabcentral/fileexchange/13664-hypsometric-integral 
 
addpath('C:\Program Files\topotoolbox-master');
addpath('C:\Program Files\topotoolbox-master\GIStools');
addpath('C:\Program Files\topotoolbox-master\colormaps');

BasinName = 'Tigum';

DEM = GRIDobj('Tigum_10m_DEM.tif');               
DEM.Z(DEM.Z < 0.01) = NaN;                        
DEM = inpaintnans(DEM);                           
FD = FLOWobj(DEM,'preprocess','carve');
DIST  = flowdistance(FD);
S  = STREAMobj(FD,'minarea',1e6,'unit','map');    
S = removeshortstreams(S,1000);                   
S = removeedgeeffects(S,FD,DEM);                  
zs50 = crs(S,DEM,'K',2,'tau',0.5);
DEMc = DEM;
DEMc.Z(S.IXgrid) = zs50;    

% Use mappingapp to select a location/outlet point (uncomment line below, run mappingapp, export point then copy x- and y- coordinate to lines below. 
%mappingapp(DEMc,S)

xTigum_outlet = 454780.511702763;       
yTigum_outlet = 1187703.72360306;       
[xn,yn,IX_out] = snap2stream(S,xTigum_outlet,yTigum_outlet,'plot',false);

DB = drainagebasins(FD,IX_out);
DIST = clip(DIST,DB);
[~,IX] = max(DIST.Z(:));
St = STREAMobj(FD,'channelheads',IX);
DB1 = drainagebasins(FD, St); 

DEMcb = clip(DEMc,DB1 > 0);

I = ~isnan(DEMcb.Z);

FDc = FLOWobj(DEMcb);
Sc = STREAMobj(FDc,'minarea',1e6,'unit','map');    
Sc = removeshortstreams(Sc,1000);                   
Sc_klarg  = klargestconncomps(Sc,1);
Sc_trunk  = trunk(Sc);
Ac  = flowacc(FDc);                              
Ac_m = (Ac.*(Ac.cellsize^2)) / 1000000;

Catchment_area_km2 = sum(sum(I) / 10000); 
Catchment_perimeter_km = bwperim(I,8); 
Catchment_perimeter_km = sum(sum(Catchment_perimeter_km)) ./ 100;

ix = streampoi(Sc_klarg,'outlets','ix');
[xout,yout] = ind2coord(DEMcb,ix);
DB  = drainagebasins(FDc,ix);
DB  = DB>0;
SEED = GRIDobj(DEMcb,'logical');
SEED.Z(ix) = true;
D    = GRIDobj(DEMcb);
D.Z  = bwdist(SEED.Z,'euclidean') * DEM.cellsize;
maxd = max(D.Z(DB.Z(:)));
LOCS = D==maxd & DB;
IX   = find(LOCS.Z);
[x,y] = ind2coord(DEMcb,IX);
Basin_length_km_euclid = (maxd./1000);

D.Z = bwdistgeodesic(DB.Z>0,SEED.Z,'q') * DEMcb.cellsize;
maxd = max(D.Z(DB.Z(:)));
LOCS = D==maxd & DB;
IX   = find(LOCS.Z);
[x,y] = ind2coord(DEMcb,IX);
Basin_length_km_constrained = (maxd./1000);

Basin_form_factor_euclid = Catchment_area_km2 / (Basin_length_km_euclid^2); % Horton 1932
Basin_form_factor_constrained = Catchment_area_km2 / (Basin_length_km_constrained^2); % Horton 1932
Basin_circularity_index = (4 * pi * (Catchment_area_km2) / (Catchment_perimeter_km ^2)); % Miller 1945
Basin_elongation_ratio_euclid = 2 * sqrt(Catchment_area_km2 / pi) / Basin_length_km_euclid;  % Schumm 1956
Basin_elongation_ratio_constrained = 2 * sqrt(Catchment_area_km2 / pi) / Basin_length_km_constrained;  % Schumm 1956
Basin_compactness_index = 0.28 * (Catchment_perimeter_km / (sqrt(Catchment_area_km2))); % Gravelius
Basin_compactness_index_2 = Catchment_perimeter_km / (2 * (sqrt(pi * Catchment_area_km2))); % Gravelius
Basin_unit_shape_factor_euclid = Basin_length_km_euclid / (sqrt(Catchment_area_km2)); % Horton 1932
Basin_unit_shape_factor_constrained = Basin_length_km_constrained / (sqrt(Catchment_area_km2)); % Horton 1932
Basin_relative_perimeter = (Catchment_area_km2 / Catchment_perimeter_km); % Schumm 1956

Elevation_mean = nanmean(DEMcb.Z,'all');
Elevation_min = min(min(DEMcb.Z));
Elevation_max = max(max(DEMcb.Z));
Elevation_relief = (Elevation_max - Elevation_min);
Elevation_10 = prctile(DEMcb.Z,10,'all');
Elevation_50 = prctile(DEMcb.Z,50,'all');
Elevation_90 = prctile(DEMcb.Z,90,'all');
Elevation_sd = nanstd(DEMcb.Z,0,'all');

Basin_relief_ratio_euclid = (Elevation_relief / Basin_length_km_euclid); % Schumm 1956
Basin_relief_ratio_constrained = (Elevation_relief / Basin_length_km_constrained); % Schumm 1956
Basin_relative_relief_ratio = (Elevation_relief / Catchment_perimeter_km);
Basin_melton_ruggedness_number = Elevation_relief / (sqrt(Catchment_area_km2));

G_catchment = gradient8(DEMcb,'deg');
G_catchment_mean = nanmean(G_catchment.Z,'all');
G_catchment_max = max(max(G_catchment.Z));
G_catchment_10 = prctile(G_catchment.Z,10,'all');
G_catchment_25 = prctile(G_catchment.Z,25,'all');
G_catchment_50 = prctile(G_catchment.Z,50,'all');
G_catchment_75 = prctile(G_catchment.Z,75,'all');
G_catchment_90 = prctile(G_catchment.Z,90,'all');
G_catchment_sd = nanstd(G_catchment.Z,0,'all');

DB_centroid_xy = drainagebasinstats(FDc,DB1);

R_TPI = roughness(DEMcb,'tpi',[101 101]); 
R_catchment_TPI_mean = nanmean(R_TPI.Z,'all');
R_catchment_TPI_min = min(min(R_TPI.Z));
R_catchment_TPI_max = max(max(R_TPI.Z));
R_catchment_TPI_10 = prctile(R_TPI.Z,10,'all');
R_catchment_TPI_50 = prctile(R_TPI.Z,50,'all');
R_catchment_TPI_90 = prctile(R_TPI.Z,90,'all');
R_catchment_TPI_sd = nanstd(R_TPI.Z,0,'all');

R_TRI = roughness(DEMcb,'tri',[101 101]); 
R_catchment_TRI_mean = nanmean(R_TRI.Z,'all');
R_catchment_TRI_min = min(min(R_TRI.Z));
R_catchment_TRI_max = max(max(R_TRI.Z));
R_catchment_TRI_10 = prctile(R_TRI.Z,10,'all');
R_catchment_TRI_50 = prctile(R_TRI.Z,50,'all');
R_catchment_TRI_90 = prctile(R_TRI.Z,90,'all');
R_catchment_TRI_sd = nanstd(R_TRI.Z,0,'all');

Catchment_hypsometric_integral = hypsometry(DEMcb.Z,100,[1 0]);

% Collate catchment characteristics
Catchment_stats = [Catchment_area_km2...
    Catchment_perimeter_km... 
    Basin_length_km_euclid...
    Basin_length_km_constrained...
    Basin_form_factor_euclid...
    Basin_form_factor_constrained...
    Basin_circularity_index...
    Basin_elongation_ratio_euclid...
    Basin_elongation_ratio_constrained...
    Basin_compactness_index...
    Basin_compactness_index_2...
    Basin_unit_shape_factor_euclid...
    Basin_unit_shape_factor_constrained...
    Basin_relative_perimeter...
    Basin_relief_ratio_euclid...
    Basin_relief_ratio_constrained...
    Basin_relative_relief_ratio...
    Basin_melton_ruggedness_number...
    Elevation_mean...
    Elevation_min...
    Elevation_max...
    Elevation_relief...
    Elevation_10...
    Elevation_50... 
    Elevation_90...
    Elevation_sd... 
    G_catchment_mean...
    G_catchment_max...
    G_catchment_10... 
    G_catchment_50... 
    G_catchment_90... 
    G_catchment_sd...
    R_catchment_TPI_mean...
    R_catchment_TPI_min...
    R_catchment_TPI_max...
    R_catchment_TPI_10... 
    R_catchment_TPI_50... 
    R_catchment_TPI_90... 
    R_catchment_TPI_sd...
    R_catchment_TRI_mean...
    R_catchment_TRI_min...
    R_catchment_TRI_max...
    R_catchment_TRI_10... 
    R_catchment_TRI_50... 
    R_catchment_TRI_90... 
    R_catchment_TRI_sd...
    Catchment_hypsometric_integral];

Catchment_stats_names = {'Catchment_area_km2'...
    'Catchment_perimeter_km'... 
    'Basin_length_km_euclid'...
    'Basin_length_km_constrained'...
    'Basin_form_factor_euclid'...
    'Basin_form_factor_constrained'...
    'Basin_circularity_index'...
    'Basin_elongation_ratio_euclid'...
    'Basin_elongation_ratio_constrained'...
    'Basin_compactness_index'...
    'Basin_compactness_index_2'...
    'Basin_unit_shape_factor_euclid'...
    'Basin_unit_shape_factor_constrained'...
    'Basin_relative_perimeter'...
    'Basin_relief_ratio_euclid'...
    'Basin_relief_ratio_constrained'...
    'Basin_relative_relief_ratio'...
    'Basin_melton_ruggedness_number'...
    'Elevation_mean'...
    'Elevation_min'...
    'Elevation_max'...
    'Elevation_relief'...
    'Elevation_10'...
    'Elevation_50'... 
    'Elevation_90'...
    'Elevation_sd'... 
    'G_catchment_mean'...
    'G_catchment_max'...
    'G_catchment_10'... 
    'G_catchment_50'... 
    'G_catchment_90'... 
    'G_catchment_sd'...
    'R_catchment_TPI_mean'...
    'R_catchment_TPI_min'...
    'R_catchment_TPI_max'...
    'R_catchment_TPI_10'... 
    'R_catchment_TPI_50'... 
    'R_catchment_TPI_90'... 
    'R_catchment_TPI_sd'...
    'R_catchment_TRI_mean'...
    'R_catchment_TRI_min'...
    'R_catchment_TRI_max'...
    'R_catchment_TRI_10'... 
    'R_catchment_TRI_50'... 
    'R_catchment_TRI_90'... 
    'R_catchment_TRI_sd'...
    'Catchment_hypsometric_integral'};

clearvars -except Catchment_stats Catchment_stats_names
