% Example MATLAB code to calculate stream network properties using TopoToolbox 

% National-scale geodatabase of catchment characteristics in the Philippines for river management applications
% February 2023 - richard.boothroyd@glasgow.ac.uk
% DOI: 10.1371/journal.pone.0281933 
% Data DOI: http://dx.doi.org/10.5525/gla.researchdata.1396   

% Example uses the Tigum catchment, the Philippines.
% Requires TopoToolbox - https://github.com/wschwanghart/topotoolbox 

% GIS shapefiles are written but need to have projection defined: WGS 1984 UTM Zone 51N (32651)

addpath('C:\Program Files\topotoolbox-master');
addpath('C:\Program Files\topotoolbox-master\GIStools');
addpath('C:\Program Files\topotoolbox-master\colormaps');

BasinName = 'Tigum';

DEM = GRIDobj('Tigum_10m_DEM.tif');               
DEM.Z(DEM.Z < 0.01) = NaN;                        
DEM = inpaintnans(DEM);                           
FD = FLOWobj(DEM,'preprocess','carve');
DIST  = flowdistance(FD);
S  = STREAMobj(FD,'minarea',1e6,'unit','map');    
S = removeshortstreams(S,1000);                   
S = removeedgeeffects(S,FD,DEM);                  
zs50 = crs(S,DEM,'K',2,'tau',0.5);
DEMc = DEM;
DEMc.Z(S.IXgrid) = zs50;    

% Use mappingapp to select a location/outlet point (uncomment line below, run mappingapp, export point then copy x- and y- coordinate to lines below. 
%mappingapp(DEMc,S)

xTigum_outlet = 454780.511702763;       
yTigum_outlet = 1187703.72360306;       
[xn,yn,IX_out] = snap2stream(S,xTigum_outlet,yTigum_outlet,'plot',false);

DB = drainagebasins(FD,IX_out);
DIST = clip(DIST,DB);
[~,IX] = max(DIST.Z(:));
St = STREAMobj(FD,'channelheads',IX);
DB1 = drainagebasins(FD, St); 

DEMcb = clip(DEMc,DB1 > 0);

I = ~isnan(DEMcb.Z);
figure;
imshow(I);

Catchment_area_km2 = sum(sum(I) / 10000); 
Catchment_perimeter_km = bwperim(I,8); 
Catchment_perimeter_km = sum(sum(Catchment_perimeter_km)) ./ 100;

FDc = FLOWobj(DEMcb);
Sc = STREAMobj(FDc,'minarea',1e6,'unit','map');    
Sc = removeshortstreams(Sc,1000);                  
Sc_klarg  = klargestconncomps(Sc,1);
Sc_trunk  = trunk(Sc_klarg);
Ac  = flowacc(FDc);                              
Ac_m = (Ac.*(Ac.cellsize^2)) / 1000000;

MS = STREAMobj2mapstruct(Sc);

Elevation_mean = nanmean(DEMcb.Z,'all');
Elevation_min = min(min(DEMcb.Z));
Elevation_max = max(max(DEMcb.Z));
Elevation_relief = (Elevation_max - Elevation_min);

SOc = streamorder(Sc);
Max_SO = max(arrayfun(@(s) max(s.streamorder), MS));

G_Sc_Tigum = gradient(Sc,DEMcb,'method','robust');
Sc_label_100m_Tigum = labelreach(Sc,'seglength',100);
Stream_slope_100m_Tigum = accumarray(Sc_label_100m_Tigum,G_Sc_Tigum,[],@mean);
Stream_slope_100m_Tigum(isnan(Stream_slope_100m_Tigum))=0.0001;
Elevc_Tigum = [getnal(Sc,DEMcb)];
Sc_elevation_Tigum = accumarray(Sc_label_100m_Tigum,Elevc_Tigum,[],@mean);

Stream_slope_100m_Tigum_p10 = prctile(Stream_slope_100m_Tigum, 10);
Stream_slope_100m_Tigum_p25 = prctile(Stream_slope_100m_Tigum, 25);
Stream_slope_100m_Tigum_p50 = prctile(Stream_slope_100m_Tigum, 50);
Stream_slope_100m_Tigum_p75 = prctile(Stream_slope_100m_Tigum, 75);
Stream_slope_100m_Tigum_p90 = prctile(Stream_slope_100m_Tigum, 90);
Stream_slope_100m_Tigum_mean = mean(Stream_slope_100m_Tigum);

Stream_length_total_km = sum(distance(Sc,'node_to_node'))/1000; 
Stream_length_mean_km = mean(distance(Sc,'from_outlet'))/1000; 
Stream_length_trunk_km = max(distance(Sc_trunk,'max_from_ch')) / 1000; 
Stream_segment_trunk = networksegment(Sc_trunk,FDc,DEMcb,Ac,0.45);
Stream_length_trunk_euclidean_distance_km = (Stream_segment_trunk.length) / 1000; 
Channel_index = Stream_length_trunk_km / Stream_length_trunk_euclidean_distance_km; 

Sc_so1 = modify(Sc,'streamorder',1);
Sc_so2 = modify(Sc,'streamorder',2);
Sc_so3 = modify(Sc,'streamorder',3);
Sc_so4 = modify(Sc,'streamorder',4);
%Sc_so5 = modify(Sc,'streamorder',5);
%Sc_so6 = modify(Sc,'streamorder',6);
%Sc_so7 = modify(Sc,'streamorder',7);
%Sc_so8 = modify(Sc,'streamorder',8);

G_Sc = gradient(Sc,DEMcb,'method','robust');
G_Sc_so1 = gradient(Sc_so1,DEMcb,'method','robust');
G_Sc_so2 = gradient(Sc_so2,DEMcb,'method','robust');
G_Sc_so3 = gradient(Sc_so3,DEMcb,'method','robust');
G_Sc_so4 = gradient(Sc_so4,DEMcb,'method','robust');
%G_Sc_so5 = gradient(Sc_so5,DEMcb,'method','robust');
%G_Sc_so6 = gradient(Sc_so6,DEMcb,'method','robust');
%G_Sc_so7 = gradient(Sc_so7,DEMcb,'method','robust');
%G_Sc_so8 = gradient(Sc_so8,DEMcb,'method','robust');

Sc_label_50m = labelreach(Sc,'seglength',50);
Stream_slope_50m = accumarray(Sc_label_50m,G_Sc,[],@mean);
Stream_slope_50m(isnan(Stream_slope_50m))=0.0001;
Stream_mean_slope_50m = nanmean(Stream_slope_50m);
Stream_median_slope_50m = nanmedian(Stream_slope_50m);
Stream_std_slope_50m = nanstd(Stream_slope_50m);

Sc_so1_label_50m = labelreach(Sc_so1,'seglength',50);
Stream_so1_slope_50m = accumarray(Sc_so1_label_50m ,G_Sc_so1,[],@mean);
Stream_so1_slope_50m(isnan(Stream_so1_slope_50m))=0.0001;
Stream_mean_slope_so1_50m = nanmean(Stream_so1_slope_50m);
Stream_median_slope_so1_50m = nanmedian(Stream_so1_slope_50m);
Stream_std_slope_so1_50m = nanstd(Stream_so1_slope_50m);

Sc_so2_label_50m = labelreach(Sc_so2,'seglength',50);
Stream_so2_slope_50m = accumarray(Sc_so2_label_50m ,G_Sc_so2,[],@mean);
Stream_so2_slope_50m(isnan(Stream_so2_slope_50m))=0.0001;
Stream_mean_slope_so2_50m = nanmean(Stream_so2_slope_50m);
Stream_median_slope_so2_50m = nanmedian(Stream_so2_slope_50m);
Stream_std_slope_so2_50m = nanstd(Stream_so2_slope_50m);

Sc_so3_label_50m = labelreach(Sc_so3,'seglength',50);
Stream_so3_slope_50m = accumarray(Sc_so3_label_50m ,G_Sc_so3,[],@mean);
Stream_so3_slope_50m(isnan(Stream_so3_slope_50m))=0.0001;
Stream_mean_slope_so3_50m = nanmean(Stream_so3_slope_50m);
Stream_median_slope_so3_50m = nanmedian(Stream_so3_slope_50m);
Stream_std_slope_so3_50m = nanstd(Stream_so3_slope_50m);

Sc_so4_label_50m = labelreach(Sc_so4,'seglength',50);
Stream_so4_slope_50m = accumarray(Sc_so4_label_50m ,G_Sc_so4,[],@mean);
Stream_so4_slope_50m(isnan(Stream_so4_slope_50m))=0.0001;
Stream_mean_slope_so4_50m = nanmean(Stream_so4_slope_50m);
Stream_median_slope_so4_50m = nanmedian(Stream_so4_slope_50m);
Stream_std_slope_so4_50m = nanstd(Stream_so4_slope_50m);

Stream_mean_slope_so5_50m = NaN; 
Stream_median_slope_so5_50m = NaN;
Stream_std_slope_so5_50m = NaN;
%Sc_so5_label_50m = labelreach(Sc_so5,'seglength',50);
%Stream_so5_slope_50m = accumarray(Sc_so5_label_50m ,G_Sc_so5,[],@mean);
%Stream_so5_slope_50m(isnan(Stream_so5_slope_50m))=0.0001;
%Stream_mean_slope_so5_50m = nanmean(Stream_so5_slope_50m);
%Stream_median_slope_so5_50m = nanmedian(Stream_so5_slope_50m);
%Stream_std_slope_so5_50m = nanstd(Stream_so5_slope_50m);

Stream_mean_slope_so6_50m = NaN; 
Stream_median_slope_so6_50m = NaN;
Stream_std_slope_so6_50m = NaN;
% Sc_so6_label_50m = labelreach(Sc_so6,'seglength',50);
% Stream_so6_slope_50m = accumarray(Sc_so6_label_50m ,G_Sc_so6,[],@mean);
% Stream_so6_slope_50m(isnan(Stream_so6_slope_50m))=0.0001;
% Stream_mean_slope_so6_50m = nanmean(Stream_so6_slope_50m);
% Stream_median_slope_so6_50m = nanmedian(Stream_so6_slope_50m);
% Stream_std_slope_so6_50m = nanstd(Stream_so6_slope_50m);

Stream_mean_slope_so7_50m = NaN; 
Stream_median_slope_so7_50m = NaN;
Stream_std_slope_so7_50m = NaN;
% Sc_so7_label_50m = labelreach(Sc_so7,'seglength',50);
% Stream_so7_slope_50m = accumarray(Sc_so7_label_50m ,G_Sc_so7,[],@mean);
% Stream_so7_slope_50m(isnan(Stream_so7_slope_50m))=0.0001;
% Stream_mean_slope_so7_50m = nanmean(Stream_so7_slope_50m);
% Stream_median_slope_so7_50m = nanmedian(Stream_so7_slope_50m);
% Stream_std_slope_so7_50m = nanstd(Stream_so7_slope_50m);

Stream_mean_slope_so8_50m = NaN; 
Stream_median_slope_so8_50m = NaN;
Stream_std_slope_so8_50m = NaN;
% Sc_so8_label_50m = labelreach(Sc_so8,'seglength',50);
% Stream_so8_slope_50m = accumarray(Sc_so8_label_50m ,G_Sc_so8,[],@mean);
% Stream_so8_slope_50m(isnan(Stream_so8_slope_50m))=0.0001;
% Stream_mean_slope_so8_50m = nanmean(Stream_so8_slope_50m);
% Stream_median_slope_so8_50m = nanmedian(Stream_so8_slope_50m);
% Stream_std_slope_so8_50m = nanstd(Stream_so8_slope_50m);

Sc_label_100m = labelreach(Sc,'seglength',100);
Stream_slope_100m = accumarray(Sc_label_100m,G_Sc,[],@mean);
Stream_slope_100m(isnan(Stream_slope_100m))=0.0001;
Stream_mean_slope_100m = nanmean(Stream_slope_100m);
Stream_median_slope_100m = nanmedian(Stream_slope_100m);
Stream_std_slope_100m = nanstd(Stream_slope_100m); 

Sc_so1_label_100m = labelreach(Sc_so1,'seglength',100);
Stream_so1_slope_100m = accumarray(Sc_so1_label_100m ,G_Sc_so1,[],@mean);
Stream_so1_slope_100m(isnan(Stream_so1_slope_100m))=0.0001;
Stream_mean_slope_so1_100m = nanmean(Stream_so1_slope_100m);
Stream_median_slope_so1_100m = nanmedian(Stream_so1_slope_100m);
Stream_std_slope_so1_100m = nanstd(Stream_so1_slope_100m); 

Sc_so2_label_100m = labelreach(Sc_so2,'seglength',100);
Stream_so2_slope_100m = accumarray(Sc_so2_label_100m ,G_Sc_so2,[],@mean);
Stream_so2_slope_100m(isnan(Stream_so2_slope_100m))=0.0001;
Stream_mean_slope_so2_100m = nanmean(Stream_so2_slope_100m);
Stream_median_slope_so2_100m = nanmedian(Stream_so2_slope_100m);
Stream_std_slope_so2_100m = nanstd(Stream_so2_slope_100m); 

Sc_so3_label_100m = labelreach(Sc_so3,'seglength',100);
Stream_so3_slope_100m = accumarray(Sc_so3_label_100m ,G_Sc_so3,[],@mean);
Stream_so3_slope_100m(isnan(Stream_so3_slope_100m))=0.0001;
Stream_mean_slope_so3_100m = nanmean(Stream_so3_slope_100m);
Stream_median_slope_so3_100m = nanmedian(Stream_so3_slope_100m);
Stream_std_slope_so3_100m = nanstd(Stream_so3_slope_100m); 
 
Sc_so4_label_100m = labelreach(Sc_so4,'seglength',100);
Stream_so4_slope_100m = accumarray(Sc_so4_label_100m ,G_Sc_so4,[],@mean);
Stream_so4_slope_100m(isnan(Stream_so4_slope_100m))=0.0001;
Stream_mean_slope_so4_100m = nanmean(Stream_so4_slope_100m);
Stream_median_slope_so4_100m = nanmedian(Stream_so4_slope_100m);
Stream_std_slope_so4_100m = nanstd(Stream_so4_slope_100m); 
 
Stream_mean_slope_so5_100m = NaN; 
Stream_median_slope_so5_100m = NaN;
Stream_std_slope_so5_100m = NaN;
%Sc_so5_label_100m = labelreach(Sc_so5,'seglength',100);
%Stream_so5_slope_100m = accumarray(Sc_so5_label_100m ,G_Sc_so5,[],@mean);
%Stream_so5_slope_100m(isnan(Stream_so5_slope_100m))=0.0001;
%Stream_mean_slope_so5_100m = nanmean(Stream_so5_slope_100m);
%Stream_median_slope_so5_100m = nanmedian(Stream_so5_slope_100m);
%Stream_std_slope_so5_100m = nanstd(Stream_so5_slope_100m); 

Stream_mean_slope_so6_100m = NaN; 
Stream_median_slope_so6_100m = NaN;
Stream_std_slope_so6_100m = NaN;
% Sc_so6_label_100m = labelreach(Sc_so6,'seglength',100);
% Stream_so6_slope_100m = accumarray(Sc_so6_label_100m ,G_Sc_so6,[],@mean);
% Stream_so6_slope_100m(isnan(Stream_so6_slope_100m))=0.0001;
% Stream_mean_slope_so6_100m = nanmean(Stream_so6_slope_100m);
% Stream_median_slope_so6_100m = nanmedian(Stream_so6_slope_100m);
% Stream_std_slope_so6_100m = nanstd(Stream_so6_slope_100m); 

Stream_mean_slope_so7_100m = NaN; 
Stream_median_slope_so7_100m = NaN;
Stream_std_slope_so7_100m = NaN; 
% Sc_so7_label_100m = labelreach(Sc_so7,'seglength',100);
% Stream_so7_slope_100m = accumarray(Sc_so7_label_100m ,G_Sc_so7,[],@mean);
% Stream_so7_slope_100m(isnan(Stream_so7_slope_100m))=0.0001;
% Stream_mean_slope_so7_100m = nanmean(Stream_so7_slope_100m);
% Stream_median_slope_so7_100m = nanmedian(Stream_so7_slope_100m);
% Stream_std_slope_so7_100m = nanstd(Stream_so7_slope_100m); 

Stream_mean_slope_so8_100m = NaN; 
Stream_median_slope_so8_100m = NaN;
Stream_std_slope_so8_100m = NaN; 
% Sc_so8_label_100m = labelreach(Sc_so8,'seglength',100);
% Stream_so8_slope_100m = accumarray(Sc_so8_label_100m ,G_Sc_so8,[],@mean);
% Stream_so8_slope_100m(isnan(Stream_so8_slope_100m))=0.0001;
% Stream_mean_slope_so8_100m = nanmean(Stream_so8_slope_100m);
% Stream_median_slope_so8_100m = nanmedian(Stream_so8_slope_100m);
% Stream_std_slope_so8_100m = nanstd(Stream_so8_slope_100m); 

Sc_label_200m = labelreach(Sc,'seglength',200);
Stream_slope_200m = accumarray(Sc_label_200m,G_Sc,[],@mean);
Stream_slope_200m(isnan(Stream_slope_200m))=0.0001;
Stream_mean_slope_200m = nanmean(Stream_slope_200m);
Stream_median_slope_200m = nanmedian(Stream_slope_200m);
Stream_std_slope_200m = nanstd(Stream_slope_200m); 

Sc_so1_label_200m = labelreach(Sc_so1,'seglength',200);
Stream_so1_slope_200m = accumarray(Sc_so1_label_200m ,G_Sc_so1,[],@mean);
Stream_so1_slope_200m(isnan(Stream_so1_slope_200m))=0.0001;
Stream_mean_slope_so1_200m = nanmean(Stream_so1_slope_200m);
Stream_median_slope_so1_200m = nanmedian(Stream_so1_slope_200m);
Stream_std_slope_so1_200m = nanstd(Stream_so1_slope_200m);
 
Sc_so2_label_200m = labelreach(Sc_so2,'seglength',200);
Stream_so2_slope_200m = accumarray(Sc_so2_label_200m ,G_Sc_so2,[],@mean);
Stream_so2_slope_200m(isnan(Stream_so2_slope_200m))=0.0001;
Stream_mean_slope_so2_200m = nanmean(Stream_so2_slope_200m);
Stream_median_slope_so2_200m = nanmedian(Stream_so2_slope_200m);
Stream_std_slope_so2_200m = nanstd(Stream_so2_slope_200m); 

Sc_so3_label_200m = labelreach(Sc_so3,'seglength',200);
Stream_so3_slope_200m = accumarray(Sc_so3_label_200m ,G_Sc_so3,[],@mean);
Stream_so3_slope_200m(isnan(Stream_so3_slope_200m))=0.0001;
Stream_mean_slope_so3_200m = nanmean(Stream_so3_slope_200m);
Stream_median_slope_so3_200m = nanmedian(Stream_so3_slope_200m);
Stream_std_slope_so3_200m = nanstd(Stream_so3_slope_200m);
 
Sc_so4_label_200m = labelreach(Sc_so4,'seglength',200);
Stream_so4_slope_200m = accumarray(Sc_so4_label_200m ,G_Sc_so4,[],@mean);
Stream_so4_slope_200m(isnan(Stream_so4_slope_200m))=0.0001;
Stream_mean_slope_so4_200m = nanmean(Stream_so4_slope_200m);
Stream_median_slope_so4_200m = nanmedian(Stream_so4_slope_200m);
Stream_std_slope_so4_200m = nanstd(Stream_so4_slope_200m);
 
Stream_mean_slope_so5_200m = NaN; 
Stream_median_slope_so5_200m = NaN;
Stream_std_slope_so5_200m = NaN; 
%Sc_so5_label_200m = labelreach(Sc_so5,'seglength',200);
%Stream_so5_slope_200m = accumarray(Sc_so5_label_200m ,G_Sc_so5,[],@mean);
%Stream_so5_slope_200m(isnan(Stream_so5_slope_200m))=0.0001;
%Stream_mean_slope_so5_200m = nanmean(Stream_so5_slope_200m);
%Stream_median_slope_so5_200m = nanmedian(Stream_so5_slope_200m);
%Stream_std_slope_so5_200m = nanstd(Stream_so5_slope_200m);

Stream_mean_slope_so6_200m = NaN; 
Stream_median_slope_so6_200m = NaN;
Stream_std_slope_so6_200m = NaN; 
% Sc_so6_label_200m = labelreach(Sc_so6,'seglength',200);
% Stream_so6_slope_200m = accumarray(Sc_so6_label_200m ,G_Sc_so6,[],@mean);
% Stream_so6_slope_200m(isnan(Stream_so6_slope_200m))=0.0001;
% Stream_mean_slope_so6_200m = nanmean(Stream_so6_slope_200m);
% Stream_median_slope_so6_200m = nanmedian(Stream_so6_slope_200m);
% Stream_std_slope_so6_200m = nanstd(Stream_so6_slope_200m);
 
Stream_mean_slope_so7_200m = NaN; 
Stream_median_slope_so7_200m = NaN;
Stream_std_slope_so7_200m = NaN; 
% Sc_so7_label_200m = labelreach(Sc_so7,'seglength',200);
% Stream_so7_slope_200m = accumarray(Sc_so7_label_200m ,G_Sc_so7,[],@mean);
% Stream_so7_slope_200m(isnan(Stream_so7_slope_200m))=0.0001;
% Stream_mean_slope_so7_200m = nanmean(Stream_so7_slope_200m);
% Stream_median_slope_so7_200m = nanmedian(Stream_so7_slope_200m);
% Stream_std_slope_so7_200m = nanstd(Stream_so7_slope_200m);

Stream_mean_slope_so8_200m = NaN; 
Stream_median_slope_so8_200m = NaN;
Stream_std_slope_so8_200m = NaN; 
% Sc_so8_label_200m = labelreach(Sc_so8,'seglength',200);
% Stream_so8_slope_200m = accumarray(Sc_so8_label_200m ,G_Sc_so8,[],@mean);
% Stream_so8_slope_200m(isnan(Stream_so8_slope_200m))=0.0001;
% Stream_mean_slope_so8_200m = nanmean(Stream_so8_slope_200m);
% Stream_median_slope_so8_200m = nanmedian(Stream_so8_slope_200m);
% Stream_std_slope_so8_200m = nanstd(Stream_so8_slope_200m);

% Segment by segment analysis, i.e. so1, so2, so3 etc.
Stream_segment_all = networksegment(Sc,FDc,DEMcb,Ac,0.45);
Stream_segment_so1 = networksegment(Sc_so1,FDc,DEMcb,Ac,0.45);
Stream_segment_so2 = networksegment(Sc_so2,FDc,DEMcb,Ac,0.45);
Stream_segment_so3 = networksegment(Sc_so3,FDc,DEMcb,Ac,0.45);
Stream_segment_so4 = networksegment(Sc_so4,FDc,DEMcb,Ac,0.45);
%Stream_segment_so5 = networksegment(Sc_so5,FDc,DEMcb,Ac,0.45);
%Stream_segment_so6 = networksegment(Sc_so6,FDc,DEMcb,Ac,0.45);
%Stream_segment_so7 = networksegment(Sc_so7,FDc,DEMcb,Ac,0.45);
%Stream_segment_so8 = networksegment(Sc_so8,FDc,DEMcb,Ac,0.45);

Stream_number_so1 = (Stream_segment_so1.n);
Stream_number_so2 = (Stream_segment_so2.n);
Stream_number_so3 = (Stream_segment_so3.n);
Stream_number_so4 = (Stream_segment_so4.n);
Stream_number_so5 = NaN;
%Stream_number_so5 = (Stream_segment_so5.n);
Stream_number_so6 = NaN;
%Stream_number_so6 = (Stream_segment_so6.n);
Stream_number_so7 = NaN;
%Stream_number_so7 = (Stream_segment_so7.n);
Stream_number_so8 = NaN;
%Stream_number_so8 = (Stream_segment_so8.n);

Stream_number_all = (Stream_number_so1 + Stream_number_so2 + Stream_number_so3 + Stream_number_so4);
%Stream_number_all = (Stream_number_so1 + Stream_number_so2 + Stream_number_so3 + Stream_number_so4 + Stream_number_so5 + Stream_number_so6);
%Stream_number_all = (Stream_number_so1 + Stream_number_so2 + Stream_number_so3 + Stream_number_so4 + Stream_number_so5 + Stream_number_so6 + Stream_number_so7);
%Stream_number_all = (Stream_number_so1 + Stream_number_so2 + Stream_number_so3 + Stream_number_so4 + Stream_number_so5 + Stream_number_so6 + Stream_number_so7 + Stream_number_so8);

Stream_length_total_km_so1 = sum(distance(Sc_so1,'node_to_node'))/1000; 
Stream_length_mean_km_so1 = mean(distance(Sc_so1,'from_outlet'))/1000; 
Stream_length_total_km_so2 = sum(distance(Sc_so2,'node_to_node'))/1000;
Stream_length_mean_km_so2 = mean(distance(Sc_so2,'from_outlet'))/1000; 
Stream_length_total_km_so3 = sum(distance(Sc_so3,'node_to_node'))/1000;
Stream_length_mean_km_so3 = mean(distance(Sc_so3,'from_outlet'))/1000; 
Stream_length_total_km_so4 = sum(distance(Sc_so4,'node_to_node'))/1000;
Stream_length_mean_km_so4 = mean(distance(Sc_so4,'from_outlet'))/1000; 
Stream_length_total_km_so5 = NaN; 
%Stream_length_total_km_so5 = sum(distance(Sc_so5,'node_to_node'))/1000; 
Stream_length_mean_km_so5 = NaN;
%Stream_length_mean_km_so5 = mean(distance(Sc_so5,'from_outlet'))/1000; 
Stream_length_total_km_so6 = NaN; 
%Stream_length_total_km_so6 = sum(distance(Sc_so6,'node_to_node'))/1000; 
Stream_length_mean_km_so6 = NaN;
%Stream_length_mean_km_so6 = mean(distance(Sc_so6,'from_outlet'))/1000; 
Stream_length_total_km_so7 = NaN; 
%Stream_length_total_km_so7 = sum(distance(Sc_so7,'node_to_node'))/1000; 
Stream_length_mean_km_so7 = NaN;
%Stream_length_mean_km_so7 = mean(distance(Sc_so7,'from_outlet'))/1000; 
Stream_length_total_km_so8 = NaN; 
%Stream_length_total_km_so8 = sum(distance(Sc_so8,'node_to_node'))/1000; 
Stream_length_mean_km_so8 = NaN;
%Stream_length_mean_km_so8 = mean(distance(Sc_so8,'from_outlet'))/1000; 

Stream_length_ratio_so1_so2_km = (Stream_length_total_km_so2 / Stream_length_total_km_so1);
Stream_length_ratio_so2_so3_km = (Stream_length_total_km_so3 / Stream_length_total_km_so2);
Stream_length_ratio_so3_so4_km = (Stream_length_total_km_so4 / Stream_length_total_km_so3);
Stream_length_ratio_mean = (Stream_length_ratio_so1_so2_km + Stream_length_ratio_so2_so3_km + Stream_length_ratio_so3_so4_km) / 3;
%Stream_length_ratio_so4_so5_km = (Stream_length_total_km_so5 / Stream_length_total_km_so4);
%Stream_length_ratio_so5_so6_km = (Stream_length_total_km_so6 / Stream_length_total_km_so5);
%Stream_length_ratio_mean = (Stream_length_ratio_so1_so2_km + Stream_length_ratio_so2_so3_km + Stream_length_ratio_so3_so4_km + Stream_length_ratio_so4_so5_km + Stream_length_ratio_so5_so6_km) / 5
%Stream_length_ratio_so6_so7_km = (Stream_length_total_km_so7 / Stream_length_total_km_so6);
%Stream_length_ratio_mean = (Stream_length_ratio_so1_so2_km + Stream_length_ratio_so2_so3_km + Stream_length_ratio_so3_so4_km + Stream_length_ratio_so4_so5_km + Stream_length_ratio_so5_so6_km + Stream_length_ratio_so6_so7_km) / 6
%Stream_length_ratio_so7_so8_km = (Stream_length_total_km_so8 / Stream_length_total_km_so7);
%Stream_length_ratio_mean = (Stream_length_ratio_so1_so2_km + Stream_length_ratio_so2_so3_km + Stream_length_ratio_so3_so4_km + Stream_length_ratio_so4_so5_km + Stream_length_ratio_so5_so6_km + Stream_length_ratio_so6_so7_km + Stream_length_ratio_so7_so8_km) / 7

Stream_bifurcation_ratio_so1_so2 = (Stream_number_so1 / Stream_number_so2); 
Stream_bifurcation_ratio_so2_so3 = (Stream_number_so2 / Stream_number_so3);
Stream_bifurcation_ratio_so3_so4 = (Stream_number_so3 / Stream_number_so4);
Stream_bifurcation_ratio_so5_so6 = NaN;
%Stream_bifurcation_ratio_so4_so5 = (Stream_number_so4 / Stream_number_so5);
Stream_bifurcation_ratio_so4_so5 = NaN;
%Stream_bifurcation_ratio_so5_so6 = (Stream_number_so5 / Stream_number_so6);
Stream_bifurcation_ratio_so6_so7 = NaN;
%Stream_bifurcation_ratio_so6_so7 = (Stream_number_so6 / Stream_number_so7);
Stream_bifurcation_ratio_so7_so8 = NaN;
%Stream_bifurcation_ratio_so7_so8 = (Stream_number_so7 / Stream_number_so8);

Stream_rho_coefficient_so1_so2 = (Stream_length_ratio_so1_so2_km / Stream_bifurcation_ratio_so1_so2);
Stream_rho_coefficient_so2_so3 = (Stream_length_ratio_so2_so3_km / Stream_bifurcation_ratio_so2_so3);
Stream_rho_coefficient_so3_so4 = (Stream_length_ratio_so3_so4_km / Stream_bifurcation_ratio_so3_so4);
%Stream_rho_coefficient_so4_so5 = (Stream_length_ratio_so4_so5_km / Stream_bifurcation_ratio_so4_so5);
%Stream_rho_coefficient_so5_so6 = (Stream_length_ratio_so5_so6_km / Stream_bifurcation_ratio_so5_so6);
%Stream_rho_coefficient_so6_so7 = (Stream_length_ratio_so6_so7_km / Stream_bifurcation_ratio_so6_so7);
%Stream_rho_coefficient_so7_so8 = (Stream_length_ratio_so7_so8_km / Stream_bifurcation_ratio_so7_so8);

MS = STREAMobj2mapstruct(Sc);
so = [MS.streamorder];
ct = 0;
clear s sn
for i = min(so):max(so)
ix = so==i;
if nnz(ix)>0
ct = ct+1;
s(ct) = i;
sn(ct) = nnz(ix);
end
end
cumsn = fliplr(cumsum(fliplr(sn)));

bf = mean(sn(1:end-1)./sn(2:end));
bfsig = std(sn(1:end-1)./sn(2:end));

Stream_mean_bifurcation_ratio = bf;
Stream_bifurcation_ratio_sig = bfsig;

Basin_drainage_density = (Stream_length_total_km / Catchment_area_km2);
Basin_drainage_frequency = (Stream_number_all / Catchment_area_km2);
Basin_drainage_intensity = (Basin_drainage_frequency / Basin_drainage_density);
Basin_constant_channel_maintenance = (1 / Basin_drainage_density);
Basin_drainage_texture = (Stream_number_all / Catchment_perimeter_km);
Basin_fitness_ratio = (Stream_length_trunk_km / Catchment_perimeter_km);
%Basin_wandering_ratio = (Stream_length_trunk_km / Basin_length_km);
Basin_infiltration_number = (Basin_drainage_frequency * Basin_drainage_density);
Basin_length_overland_flow = (Catchment_area_km2 / (2 * Stream_length_total_km));
Basin_ruggedness_number =  (Basin_drainage_density * (Elevation_relief / 1000));

% Collate stream network characteristics
Stream_network_stats = [Max_SO...
    Stream_length_total_km...
    Stream_length_mean_km...
    Stream_length_trunk_km...
    Stream_length_trunk_euclidean_distance_km...
    Channel_index...
    Stream_mean_slope_50m...
    Stream_median_slope_50m...
    Stream_std_slope_50m...
    Stream_mean_slope_so1_50m...
    Stream_median_slope_so1_50m...
    Stream_std_slope_so1_50m...
    Stream_mean_slope_so2_50m...
    Stream_median_slope_so2_50m...
    Stream_std_slope_so2_50m...
    Stream_mean_slope_so3_50m...
    Stream_median_slope_so3_50m...
    Stream_std_slope_so3_50m...
    Stream_mean_slope_so4_50m...
    Stream_median_slope_so4_50m...
    Stream_std_slope_so4_50m...
    Stream_mean_slope_so5_50m...
    Stream_median_slope_so5_50m...
    Stream_std_slope_so5_50m...
    Stream_mean_slope_so6_50m...
    Stream_median_slope_so6_50m...
    Stream_std_slope_so6_50m...
    Stream_mean_slope_so7_50m...
    Stream_median_slope_so7_50m...
    Stream_std_slope_so7_50m...
    Stream_mean_slope_so8_50m...
    Stream_median_slope_so8_50m...
    Stream_std_slope_so8_50m...
    Stream_mean_slope_100m...
    Stream_median_slope_100m...
    Stream_std_slope_100m...
    Stream_mean_slope_so1_100m...
    Stream_median_slope_so1_100m...
    Stream_std_slope_so1_100m...
    Stream_mean_slope_so2_100m...
    Stream_median_slope_so2_100m...
    Stream_std_slope_so2_100m...
    Stream_mean_slope_so3_100m...
    Stream_median_slope_so3_100m...
    Stream_std_slope_so3_100m...
    Stream_mean_slope_so4_100m...
    Stream_median_slope_so4_100m...
    Stream_std_slope_so4_100m...
    Stream_mean_slope_so5_100m...
    Stream_median_slope_so5_100m...
    Stream_std_slope_so5_100m...
    Stream_mean_slope_so6_100m...
    Stream_median_slope_so6_100m...
    Stream_std_slope_so6_100m...
    Stream_mean_slope_so7_100m...
    Stream_median_slope_so7_100m...
    Stream_std_slope_so7_100m...
    Stream_mean_slope_so8_100m...
    Stream_median_slope_so8_100m...
    Stream_std_slope_so8_100m...
    Stream_mean_slope_200m...
    Stream_median_slope_200m...
    Stream_std_slope_200m...
    Stream_mean_slope_so1_200m...
    Stream_median_slope_so1_200m...
    Stream_std_slope_so1_200m...
    Stream_mean_slope_so2_200m...
    Stream_median_slope_so2_200m...
    Stream_std_slope_so2_200m...
    Stream_mean_slope_so3_200m...
    Stream_median_slope_so3_200m...
    Stream_std_slope_so3_200m...
    Stream_mean_slope_so4_200m...
    Stream_median_slope_so4_200m...
    Stream_std_slope_so4_200m...
    Stream_mean_slope_so5_200m...
    Stream_median_slope_so5_200m...
    Stream_std_slope_so5_200m...
    Stream_mean_slope_so6_200m...
    Stream_median_slope_so6_200m...
    Stream_std_slope_so6_200m...
    Stream_mean_slope_so7_200m...
    Stream_median_slope_so7_200m...
    Stream_std_slope_so7_200m...
    Stream_mean_slope_so8_200m...
    Stream_median_slope_so8_200m...
    Stream_std_slope_so8_200m...
    Stream_length_total_km_so1...
    Stream_length_mean_km_so1...
    Stream_length_total_km_so2...
    Stream_length_mean_km_so2...
    Stream_length_total_km_so3...
    Stream_length_mean_km_so3...
    Stream_length_total_km_so4...
    Stream_length_mean_km_so4...
    Stream_length_total_km_so5...
    Stream_length_mean_km_so5...
    Stream_length_total_km_so6...
    Stream_length_mean_km_so6...
    Stream_length_total_km_so7...
    Stream_length_mean_km_so7...
    Stream_length_total_km_so8...
    Stream_length_mean_km_so8...
    Stream_length_ratio_mean...
    Stream_number_all...
    Stream_mean_bifurcation_ratio...
    Stream_bifurcation_ratio_sig...
    Basin_drainage_density...
    Basin_drainage_frequency...
    Basin_drainage_intensity...
    Basin_constant_channel_maintenance...
    Basin_drainage_texture...
    Basin_fitness_ratio...
    Basin_infiltration_number...
    Basin_length_overland_flow...
    Basin_ruggedness_number];

Stream_network_stats_names = {'Max_SO'...
    'Stream_length_total_km'...
    'Stream_length_mean_km'...
    'Stream_length_trunk_km'...
    'Stream_length_trunk_euclidean_distance_km'...
    'Channel_index'...
    'Stream_mean_slope_50m'...
    'Stream_median_slope_50m'...
    'Stream_std_slope_50m'...
    'Stream_mean_slope_so1_50m'...
    'Stream_median_slope_so1_50m'...
    'Stream_std_slope_so1_50m'...
    'Stream_mean_slope_so2_50m'...
    'Stream_median_slope_so2_50m'...
    'Stream_std_slope_so2_50m'...
    'Stream_mean_slope_so3_50m'...
    'Stream_median_slope_so3_50m'...
    'Stream_std_slope_so3_50m'...
    'Stream_mean_slope_so4_50m'...
    'Stream_median_slope_so4_50m'...
    'Stream_std_slope_so4_50m'...
    'Stream_mean_slope_so5_50m'...
    'Stream_median_slope_so5_50m'...
    'Stream_std_slope_so5_50m'...
    'Stream_mean_slope_so6_50m'...
    'Stream_median_slope_so6_50m'...
    'Stream_std_slope_so6_50m'...
    'Stream_mean_slope_so7_50m'...
    'Stream_median_slope_so7_50m'...
    'Stream_std_slope_so7_50m'...
    'Stream_mean_slope_so8_50m'...
    'Stream_median_slope_so8_50m'...
    'Stream_std_slope_so8_50m'...
    'Stream_mean_slope_100m'...
    'Stream_median_slope_100m'...
    'Stream_std_slope_100m'...
    'Stream_mean_slope_so1_100m'...
    'Stream_median_slope_so1_100m'...
    'Stream_std_slope_so1_100m'...
    'Stream_mean_slope_so2_100m'...
    'Stream_median_slope_so2_100m'...
    'Stream_std_slope_so2_100m'...
    'Stream_mean_slope_so3_100m'...
    'Stream_median_slope_so3_100m'...
    'Stream_std_slope_so3_100m'...
    'Stream_mean_slope_so4_100m'...
    'Stream_median_slope_so4_100m'...
    'Stream_std_slope_so4_100m'...
    'Stream_mean_slope_so5_100m'...
    'Stream_median_slope_so5_100m'...
    'Stream_std_slope_so5_100m'...
    'Stream_mean_slope_so6_100m'...
    'Stream_median_slope_so6_100m'...
    'Stream_std_slope_so6_100m'...
    'Stream_mean_slope_so7_100m'...
    'Stream_median_slope_so7_100m'...
    'Stream_std_slope_so7_100m'...
    'Stream_mean_slope_so8_100m'...
    'Stream_median_slope_so8_100m'...
    'Stream_std_slope_so8_100m'...
    'Stream_mean_slope_200m'...
    'Stream_median_slope_200m'...
    'Stream_std_slope_200m'...
    'Stream_mean_slope_so1_200m'...
    'Stream_median_slope_so1_200m'...
    'Stream_std_slope_so1_200m'...
    'Stream_mean_slope_so2_200m'...
    'Stream_median_slope_so2_200m'...
    'Stream_std_slope_so2_200m'...
    'Stream_mean_slope_so3_200m'...
    'Stream_median_slope_so3_200m'...
    'Stream_std_slope_so3_200m'...
    'Stream_mean_slope_so4_200m'...
    'Stream_median_slope_so4_200m'...
    'Stream_std_slope_so4_200m'...
    'Stream_mean_slope_so5_200m'...
    'Stream_median_slope_so5_200m'...
    'Stream_std_slope_so5_200m'...
    'Stream_mean_slope_so6_200m'...
    'Stream_median_slope_so6_200m'...
    'Stream_std_slope_so6_200m'...
    'Stream_mean_slope_so7_200m'...
    'Stream_median_slope_so7_200m'...
    'Stream_std_slope_so7_200m'...
    'Stream_mean_slope_so8_200m'...
    'Stream_median_slope_so8_200m'...
    'Stream_std_slope_so8_200m'...
    'Stream_length_total_km_so1'...
    'Stream_length_mean_km_so1'...
    'Stream_length_total_km_so2'...
    'Stream_length_mean_km_so2'...
    'Stream_length_total_km_so3'...
    'Stream_length_mean_km_so3'...
    'Stream_length_total_km_so4'...
    'Stream_length_mean_km_so4'...
    'Stream_length_total_km_so5'...
    'Stream_length_mean_km_so5'...
    'Stream_length_total_km_so6'...
    'Stream_length_mean_km_so6'...
    'Stream_length_total_km_so7'...
    'Stream_length_mean_km_so7'...
    'Stream_length_total_km_so8'...
    'Stream_length_mean_km_so8'...
    'Stream_length_ratio_mean'...
    'Stream_number_all'...
    'Stream_mean_bifurcation_ratio'...
    'Stream_bifurcation_ratio_sig'...
    'Basin_drainage_density'...
    'Basin_drainage_frequency'...
    'Basin_drainage_intensity'...
    'Basin_constant_channel_maintenance'...
    'Basin_drainage_texture'...
    'Basin_fitness_ratio'...
    'Basin_infiltration_number'...
    'Basin_length_overland_flow'...
    'Basin_ruggedness_number'};


% Write outputs to GIS shapefiles:

[DB_MS,DB_x,DB_y] = GRIDobj2polygon(DB1,'Geometry','Polygon','simplify',false,'tol',0);
shapewrite(DB_MS,'Tigum_Catchment.shp');

MS_50 = STREAMobj2mapstruct(Sc,'seglength',50,'attributes',...
         {'distance_from_outlet_km' (Sc.distance / 1000) @mean...
          'upstream_area_m_2' (Ac.*(Ac.cellsize^2)) @median ...
          'gradient_m_m' G_Sc @mean ...
          'streamord' SOc @max...
          'elev' DEMc @mean});
shapewrite(MS_50,'Tigum_Stream_Network_10m_D_A_G_SO_ELV_50m.shp');

MS_100 = STREAMobj2mapstruct(Sc,'seglength',100,'attributes',...
         {'distance_from_outlet_km' (Sc.distance / 1000) @mean...
          'upstream_area_m_2' (Ac.*(Ac.cellsize^2)) @median ...
          'gradient_m_m' G_Sc @mean ...
          'streamord' SOc @max...
          'elev' DEMc @mean});
shapewrite(MS_100,'Tigum_Stream_Network_10m_D_A_G_SO_ELV_100m.shp');

MS_200 = STREAMobj2mapstruct(Sc,'seglength',200,'attributes',...
         {'distance_from_outlet_km' (Sc.distance / 1000) @mean...
          'upstream_area_m_2' (Ac.*(Ac.cellsize^2)) @median ...
          'gradient_m_m' G_Sc @mean ...
          'streamord' SOc @max...
          'elev' DEMc @mean});
shapewrite(MS_200,'Tigum_Stream_Network_10m_D_A_G_SO_ELV_200m.shp');

MS_1000 = STREAMobj2mapstruct(Sc,'seglength',1000,'attributes',...
         {'distance_from_outlet_km' (Sc.distance / 1000) @mean...
          'upstream_area_m_2' ((Ac.*(Ac.cellsize^2)) / 1000000) @median ...
          'gradient_m_m' G_Sc @mean ...
          'streamord' SOc @max...
          'elev' DEMc @mean});
shapewrite(MS_1000,'Tigum_Stream_Network_10m_D_A_G_SO_ELV_1000m.shp');

clearvars -except Stream_network_stats Stream_network_stats_names